@@ -180,16 +180,16 @@ impl Keystore {
 
         for mut b in backends.unwrap_or_else(|_| Vec::new()).into_iter() {
             let devices = if _sync {
-                b.list()
+                b.devices()
             } else {
-                b.list_async().await
+                b.devices_async().await
             };
 
             for mut d in devices.unwrap_or_else(|_| Vec::new()).into_iter() {
                 let keys = if _sync {
-                    d.list()
+                    d.keys()
                 } else {
-                    d.list_async().await
+                    d.keys_async().await
                 };
 
                 for mut k in keys.unwrap_or_else(|_| Vec::new()).into_iter() {
@@ -251,7 +251,7 @@ impl Keystore {
         /// decrypted the PKESK, and the plaintext (the symmetric
         /// algorithm and the session key).
         fn [keystore]decrypt/1(&mut self, pkesks: &[PKESK])
-            -> Result<(usize, Fingerprint, SymmetricAlgorithm, SessionKey)>
+            -> Result<(usize, Fingerprint, Option<SymmetricAlgorithm>, SessionKey)>
         // Marshal.
         |root: keystore::decrypt_params::Builder| -> Result<()> {
             // Convert the arguments into the form expected by the RPC.
@@ -276,11 +276,16 @@ impl Keystore {
                 // The RPC's result:
                 Which::Ok(Ok(r)) => {
                     let i = r.get_index() as usize;
-                    let fpr = Fingerprint::from_bytes(r.get_fingerprint()?);
-                    let algo = SymmetricAlgorithm::from(r.get_algo() as u8);
+                    let fp_version = r.get_fingerprint_version();
+                    let fpr = Fingerprint::from_bytes(
+                        fp_version, r.get_fingerprint()?)?;
+                    let algo = match r.get_algo() as u8 {
+                        0 => None,
+                        n => Some(n.into()),
+                    };
                     let session_key = SessionKey::from(r.get_session_key()?);
 
-                    let r: (usize, Fingerprint, SymmetricAlgorithm, SessionKey)
+                    let r: (usize, Fingerprint, Option<SymmetricAlgorithm>, SessionKey)
                         = (i, fpr, algo, session_key);
                     Ok(r)
                 },
@@ -337,15 +342,15 @@ impl Backend {
         /// Lists all devices.
         ///
         /// Lists the devices managed by a backend.
-        fn [keystore::backend] list/1(&mut self)
+        fn [keystore::backend] devices/1(&mut self)
             -> Result<Vec<Cap>>
             -> Result<Vec<Device>>
         // Marshal.
-        |_root: keystore::backend::list_params::Builder| -> Result<()> {
+        |_root: keystore::backend::devices_params::Builder| -> Result<()> {
             Ok(())
         };
         // Extract.
-        |relay, captable, response: keystore::backend::list_results::Reader| {
+        |relay, captable, response: keystore::backend::devices_results::Reader| {
             use keystore::result::Which;
             match response.get_result()?.which()? {
                 // The RPC's result:
@@ -528,17 +533,17 @@ impl Device {
         /// Some of the returned keys may be known, but not currently
         /// available.  For instance, if a smartcard is not plugged
         /// in, or an ssh connection is not established.
-        fn [keystore::device] list/1(&mut self)
+        fn [keystore::device] keys/1(&mut self)
             -> Result<Vec<(Cap,
                            packet::Key::<key::PublicParts,
                                          key::UnspecifiedRole>)>>
             -> Result<Vec<Key>>
         // Marshal.
-        |_root: keystore::device::list_params::Builder| -> Result<()> {
+        |_root: keystore::device::keys_params::Builder| -> Result<()> {
             Ok(())
         };
         // Extract.
-        |relay, captable, response: keystore::device::list_results::Reader| {
+        |relay, captable, response: keystore::device::keys_results::Reader| {
             use keystore::result::Which;
             match response.get_result()?.which()? {
                 // The RPC's result:
@@ -635,6 +640,11 @@ pub struct Key {
 }
 
 impl Key {
+    /// Returns the key's key handle.
+    pub fn key_handle(&self) -> KeyHandle {
+        self.key.fingerprint().into()
+    }
+
     /// Returns the key's fingerprint.
     pub fn fingerprint(&self) -> Fingerprint {
         self.key.fingerprint()
@@ -1186,13 +1196,21 @@ mod tests {
             if let Packet::PKESK(pkesk) = packet {
                 saw_pkesk = true;
 
-                let recipient = KeyHandle::from(pkesk.recipient());
+                let recipient = pkesk.recipient();
 
                 let should_decrypt = recipients.iter()
-                    .find(|(r, _password)| {
-                        recipient.aliases(KeyHandle::from(r))
+                    .find(|(rcpv, _password)| {
+                        recipient.as_ref()
+                            .map(|r| r.aliases(KeyHandle::from(rcpv)))
+                            .unwrap_or(false)
                     });
 
+                let recipient = if let Some(r) = recipient {
+                    r
+                } else {
+                    continue;
+                };
+
                 let kh = match ks.find_key(recipient.clone()) {
                     Ok(kh) => kh,
                     Err(err) => {
@@ -1277,8 +1295,9 @@ mod tests {
         Ok(results)
     }
 
-    // A simple test that lists all of the keys.
+    // A simple test that lists all the keys.
     #[test]
+    #[cfg(feature = "softkeys")]
     fn simple_ping() -> Result<()> {
         let home = Home::ephemeral().unwrap();
         eprintln!("Using {}", home);
@@ -1312,13 +1331,13 @@ mod tests {
             }
 
             eprintln!(" - {}", backend.id()?);
-            let devices = backend.list()?;
+            let devices = backend.devices()?;
             if is_softkeys {
                 assert_eq!(devices.len(), 3);
             }
             for mut device in devices {
                 eprintln!("   - {}", device.id()?);
-                let keys = device.list()?;
+                let keys = device.keys()?;
                 if is_softkeys {
                     assert_eq!(keys.len(), 4);
                 }
@@ -1334,6 +1353,7 @@ mod tests {
 
     // Bash on the relay from multiple threads.
     #[test]
+    #[cfg(feature = "softkeys")]
     fn simple_bash() -> Result<()> {
         use std::thread;
         use std::sync::Mutex;
@@ -1377,7 +1397,7 @@ mod tests {
                         // away.
                         for _ in 1..3 {
                             eprintln!(" {}.{}. {}", t, b, backend.id()?);
-                            let devices = backend.list()?;
+                            let devices = backend.devices()?;
                             if is_softkeys {
                                 assert_eq!(devices.len(), 3);
                             }
@@ -1385,7 +1405,7 @@ mod tests {
                                 for _ in 1..3 {
                                     eprintln!("   {}.{}.{}. {}",
                                               t, b, d, device.id()?);
-                                    let keys = device.list()?;
+                                    let keys = device.keys()?;
                                     if is_softkeys {
                                         assert_eq!(keys.len(), 4);
                                     }
@@ -1413,6 +1433,7 @@ mod tests {
 
     // Test Keystore::decrypt using the simple keystore.
     #[test]
+    #[cfg(feature = "softkeys")]
     fn simple_keystore_decrypt() -> Result<()> {
         let home = Home::ephemeral().unwrap();
         eprintln!("Using {}", home);
@@ -1432,7 +1453,7 @@ mod tests {
 
         // Run the tests.
         let mut bad = 0;
-        for msg in simple::MSGS.iter() {
+        for msg in simple::msgs() {
             eprintln!("Decrypting {}", msg.filename);
 
             let mut pkesks = Vec::new();
@@ -1459,12 +1480,12 @@ mod tests {
                                 KeyHandle::from(r).aliases(&fpr)
                             })
                             {
-                                eprintln!("fpr ({}) is not an expected \
+                                eprintln!("fpr ({:?}) is not an expected \
                                            recipient ({})",
                                           fpr,
                                           msg.recipients
                                               .iter()
-                                              .map(|r| r.to_string())
+                                              .map(|r| format!("{:?}", r))
                                               .collect::<Vec<String>>()
                                               .join(", "));
                                 bad += 1;
@@ -1472,12 +1493,13 @@ mod tests {
                             }
 
                             // Make sure the fpr and the pkesk match.
-                            if ! fpr.aliases(
-                                KeyHandle::from(pkesks[i].recipient()))
+                            if ! pkesks[i].recipient()
+                                .map(|r| fpr.aliases(r))
+                                .unwrap_or(true)
                             {
                                 eprintln!("fpr ({}) and pkesk ({}) \
                                            don't match",
-                                          fpr, pkesks[i].recipient());
+                                          fpr, pkesks[i].recipient().unwrap());
                                 bad += 1;
                                 break;
                             }
@@ -1512,6 +1534,7 @@ mod tests {
                 let (packet, next_ppr) = pp.recurse()?;
                 ppr = next_ppr;
 
+                #[allow(deprecated)]
                 match packet {
                     Packet::PKESK(pkesk) if ! decrypted => pkesks.push(pkesk),
                     Packet::SEIP(_) if decrypted => (),
@@ -1542,6 +1565,7 @@ mod tests {
 
     // Test Key::decrypt_pkesk using the simple keystore.
     #[test]
+    #[cfg(feature = "softkeys")]
     fn simple_key_decrypt_pkesk() -> Result<()> {
         let home = Home::ephemeral().unwrap();
         eprintln!("Using {}", home);
@@ -1561,7 +1585,7 @@ mod tests {
 
         // We also try to decrypt with Alice's and Carol's primary
         // keys to make sure that that fails.
-        let alice_pri = ks.find_key(KeyHandle::from(&*simple::alice_pri))
+        let alice_pri = ks.find_key(KeyHandle::from(&simple::fp().alice_pri))
             .expect("keystore is up");
         // In the simple keystore, keys are not repeated.
         assert!(alice_pri.len() <= 1);
@@ -1571,7 +1595,7 @@ mod tests {
             panic!("Alice's primary key is not present on the key store.");
         };
 
-        let carol_pri = ks.find_key(KeyHandle::from(&*simple::carol_pri))
+        let carol_pri = ks.find_key(KeyHandle::from(&simple::fp().carol_pri))
             .expect("keystore is up");
         // In the simple keystore, keys are not repeated.
         assert!(carol_pri.len() <= 1);
@@ -1584,7 +1608,7 @@ mod tests {
 
         // Run the tests.
         let mut bad = 0;
-        for msg in simple::MSGS.iter() {
+        for msg in simple::msgs() {
             eprintln!("Decrypting {}", msg.filename);
 
             let mut saw_pkesk = false;
@@ -1601,14 +1625,22 @@ mod tests {
                 if let Packet::PKESK(pkesk) = packet {
                     saw_pkesk = true;
 
-                    let recip = KeyHandle::from(pkesk.recipient());
+                    let recip = pkesk.recipient();
 
                     let should_decrypt = msg.recipients
                         .iter()
                         .any(|&r| {
-                            KeyHandle::from(r).aliases(recip.clone())
+                            recip.as_ref()
+                                .map(|recip| recip.aliases(&KeyHandle::from(r)))
+                                .unwrap_or(false)
                         });
 
+                    let recip = if let Some(r) = recip {
+                        r
+                    } else {
+                        continue;
+                    };
+
                     let kh = ks.find_key(recip.clone())?;
                     // In the simple keystore, keys are not repeated.
                     assert!(kh.len() <= 1);
@@ -1679,6 +1711,7 @@ mod tests {
     }
 
     #[test]
+    #[cfg(feature = "softkeys")]
     fn simple_key_sign() -> Result<()> {
         let home = Home::ephemeral().unwrap();
         eprintln!("Using {}", home);
@@ -1704,7 +1737,7 @@ mod tests {
                 .iter()
                 .flat_map(|cert| cert.keys())
                 .find(|k| {
-                    k.fingerprint() == fpr
+                    k.key().fingerprint() == fpr
                 })
         };
 
@@ -1731,11 +1764,11 @@ mod tests {
                 continue;
             }
 
-            let devices = backend.list()?;
+            let devices = backend.devices()?;
             assert_eq!(devices.len(), 3);
 
             for mut device in devices {
-                let keys = device.list()?;
+                let keys = device.keys()?;
                 assert_eq!(keys.len(), 4);
 
                 for mut key in keys.into_iter() {
@@ -1743,24 +1776,24 @@ mod tests {
                         .expect("have corresponding certificate");
 
                     eprintln!("  - {} ({}, {})",
-                              ka.fingerprint(),
+                              ka.key().fingerprint(),
                               ka
                                   .cert()
                                   .userids()
                                   .next()
                                   .map(|ua| {
-                                      String::from_utf8_lossy(ua.value())
+                                      String::from_utf8_lossy(ua.userid().value())
                                           .into_owned()
                                   })
                                   .unwrap_or_else(|| {
                                       String::from("<no user ids>")
                                   }),
-                              ka.pk_algo());
+                              ka.key().pk_algo());
 
                     // Note: we don't consider keyflags, because the
                     // key store can't either (even if a backend has
                     // the whole cert, it may not be up to date).
-                    let bad_key = ! ka.pk_algo().for_signing();
+                    let bad_key = ! ka.key().pk_algo().for_signing();
 
                     let sig = SignatureBuilder::new(SignatureType::Binary);
 
@@ -1807,6 +1840,7 @@ mod tests {
 
     // Test Key::decrypt_pkesk using the password keystore.
     #[test]
+    #[cfg(feature = "softkeys")]
     fn password_key_decrypt_pkesk() -> Result<()> {
         let _ = env_logger::Builder::from_default_env().try_init();
 
@@ -1827,12 +1861,12 @@ mod tests {
         let mut ks = Keystore::connect(&c)?;
 
         // Run the tests.
-        for (msg, pw) in password::MSGS.iter() {
+        for (msg, pw) in password::msgs() {
             eprintln!("Decrypting {}", msg.filename);
 
             let content = testdata::file(msg.filename);
 
-            let recipients = msg.recipients.iter().map(|&r| (r.clone(), *pw))
+            let recipients = msg.recipients.iter().map(|&r| (r.clone(), pw))
                 .collect::<Vec<(Fingerprint, Option<&str>)>>();
             try_decrypt(
                 &mut ks, content, &recipients[..], true)
@@ -1844,6 +1878,7 @@ mod tests {
 
     // Test Key::decrypt_pkesk using the password keystore.
     #[test]
+    #[cfg(feature = "softkeys")]
     fn password_key_decrypt_pkesk2() -> Result<()> {
         let home = Home::ephemeral().unwrap();
         eprintln!("Using {}", home);
@@ -1861,7 +1896,7 @@ mod tests {
             .build()?;
         let mut ks = Keystore::connect(&c)?;
 
-        for (msg, password) in testdata::password::MSGS.iter() {
+        for (msg, password) in testdata::password::msgs() {
             eprintln!("{}", msg.filename);
 
             let pp = PacketPile::from_bytes(testdata::file(msg.filename))
@@ -1922,7 +1957,7 @@ mod tests {
                     assert!(r.is_none());
 
                     // Unlock it and try again.
-                    key.key_mut().unlock(Password::from(*password))
+                    key.key_mut().unlock(Password::from(password))
                         .expect("correct password");
 
                     let r = pkesk.decrypt(key.key_mut(), None);
@@ -1936,6 +1971,7 @@ mod tests {
 
     // Test Key::import and Key::export.
     #[test]
+    #[cfg(feature = "softkeys")]
     fn import_export() -> Result<()> {
         let home = Home::ephemeral().unwrap();
         eprintln!("Using {}", home);
@@ -1966,20 +2002,21 @@ mod tests {
             eprintln!("Testing a {:?} cert", cs);
 
             let (cert, _rev) = CertBuilder::general_purpose(
-                cs, Some("alice"))
+                Some("alice"))
+                .set_cipher_suite(cs)
                 .generate()?;
 
             softkeys.import(&cert).expect("can import");
 
             for key in cert.keys() {
-                eprintln!("Exporting {}", key.fingerprint());
+                eprintln!("Exporting {}", key.key().fingerprint());
 
                 let key = key.parts_into_secret().expect("has secret");
 
-                let remote = ks.find_key(key.key_handle()).expect("have key");
+                let remote = ks.find_key(key.key().key_handle()).expect("have key");
                 assert_eq!(remote.len(), 1, "only one key under this name");
                 let mut remote = remote.into_iter().next().unwrap();
-                assert_eq!(remote.fingerprint(), key.fingerprint());
+                assert_eq!(remote.fingerprint(), key.key().fingerprint());
 
                 let export = remote.export().expect("can export");
                 assert_eq!(&export, key.key());
@@ -1992,6 +2029,7 @@ mod tests {
 
     // Test Key::password_source.
     #[test]
+    #[cfg(feature = "softkeys")]
     fn password_source() -> Result<()> {
         let home = Home::ephemeral().unwrap();
         eprintln!("Using {}", home);
@@ -2016,7 +2054,7 @@ mod tests {
         // This is a pretty naive test: we just make sure the backend
         // returns success.
         for password in [false, true] {
-            let mut builder = CertBuilder::general_purpose(None, Some("alice"));
+            let mut builder = CertBuilder::general_purpose(Some("alice"));
             if password {
                 builder = builder.set_password(Some("foo".into()));
             }
@@ -2052,6 +2090,7 @@ mod tests {
 
     // Test Key::change_password
     #[test]
+    #[cfg(feature = "softkeys")]
     fn change_password() -> Result<()> {
         let _ = env_logger::Builder::from_default_env().try_init();
 
@@ -2072,14 +2111,14 @@ mod tests {
         let mut ks = Keystore::connect(&c)?;
 
         // Run the tests.
-        for (msg, pw) in password::MSGS.iter() {
+        for (msg, pw) in password::msgs() {
             eprintln!("Decrypting {}", msg.filename);
 
             let content = testdata::file(msg.filename);
 
             let new_password = "a NEW pa$$word";
 
-            let recipients = msg.recipients.iter().map(|&r| (r.clone(), *pw))
+            let recipients = msg.recipients.iter().map(|&r| (r.clone(), pw))
                 .collect::<Vec<(Fingerprint, Option<&str>)>>();
 
             try_decrypt(
@@ -2097,7 +2136,7 @@ mod tests {
                         panic!("Should have a key for {}", r);
                     };
 
-                    if let Some(pw) = *pw {
+                    if let Some(pw) = pw {
                         if let Protection::Password(_hint)
                             = kh.locked().expect("locked")
                         {
