@@ -18,6 +18,7 @@ use openpgp::packet::key;
 use openpgp::parse::Parse;
 use openpgp::types::HashAlgorithm;
 use openpgp::types::PublicKeyAlgorithm;
+use openpgp::Fingerprint;
 
 use sequoia_ipc as ipc;
 
@@ -89,7 +90,6 @@ struct KeystoreServer {
 
 struct KeystoreServerInner {
     home: PathBuf,
-    rt: tokio::runtime::Runtime,
 }
 
 impl std::fmt::Debug for KeystoreServerInner {
@@ -106,7 +106,6 @@ impl KeystoreServer {
         Ok(Self {
             inner: Arc::new(KeystoreServerInner {
                 home,
-                rt: tokio::runtime::Runtime::new()?,
             }),
         })
     }
@@ -151,8 +150,8 @@ macro_rules! run {
             // Phase 2: Execute the method's body in a separate task
             // so that tasks that are bound to this thread are not
             // blocked.
-            let ks: KeystoreServer = $ks;
-            let r: JoinResult<Result<_>> = ks.inner.rt.spawn($phase2).await;
+            let _ks: KeystoreServer = $ks;
+            let r: JoinResult<Result<_>> = tokio::task::spawn($phase2).await;
 
             // Convert a join error to an internal server error.
             let r = match r {
@@ -242,7 +241,10 @@ impl keystore::Server for KeystoreServer {
         log::trace!("KeystoreServer::decrypt({})",
                     pkesks.iter()
                     .map(|(_i, pkesk)| {
-                        pkesk.recipient().to_string()
+                        pkesk.recipient()
+                            .map(|r| r.to_string())
+                            .unwrap_or_else(
+                                || "anonymous".into())
                     })
                     .collect::<Vec<String>>()
                     .join(", "));
@@ -257,7 +259,7 @@ impl keystore::Server for KeystoreServer {
                 let (wildcard, known): (Vec<&(usize, PKESK)>, Vec<&(usize, PKESK)>)
                     = pkesks
                     .iter()
-                    .partition(|(_i, pkesk)| pkesk.recipient().is_wildcard());
+                    .partition(|(_i, pkesk)| pkesk.recipient().is_none());
 
                 // Keys that are inaccessible, but that could become available
                 // with a little help from the user (e.g., unlock a password,
@@ -268,13 +270,17 @@ impl keystore::Server for KeystoreServer {
                 let mut server = server.lock().await;
 
                 if ! known.is_empty() {
+                    log::trace!("Considering PKESKs with known recipients");
                     for backend in server.backends.iter_mut() {
                         for device in backend.list().await {
                             for mut key in device.list().await {
                                 for (i, pkesk) in known.iter() {
-                                    let keyid = key.keyid();
-                                    if pkesk.recipient() != &keyid {
-                                        log::trace!("PKESK for {}, key: {}, skipping",
+                                    let keyid = key.key_handle();
+                                    if ! pkesk.recipient()
+                                        .map(|h| h.aliases(&keyid))
+                                        .unwrap_or(false)
+                                    {
+                                        log::trace!("PKESK for {:?}, key: {}, skipping",
                                                     pkesk.recipient(), keyid);
                                         continue;
                                     }
@@ -328,10 +334,10 @@ impl keystore::Server for KeystoreServer {
                                     if let Some((sym_algo, sk))
                                         = key.decrypt_pkesk(pkesk).await
                                     {
-                                        let fpr = key.fingerprint().as_bytes().to_vec();
+                                        let fpr = key.fingerprint();
                                         return Ok((*i, fpr, sym_algo, sk));
                                     } else {
-                                        log::trace!("PKESK for {} matches key {}, \
+                                        log::trace!("PKESK for {:?} matches key {}, \
                                                      but failed to decrypt it!",
                                                     pkesk.recipient(), keyid);
                                     }
@@ -342,7 +348,78 @@ impl keystore::Server for KeystoreServer {
                 }
 
                 if ! wildcard.is_empty() {
-                    unimplemented!();
+                    log::trace!("Considering PKESKs with anonymous recipients");
+                    for backend in server.backends.iter_mut() {
+                        for device in backend.list().await {
+                            for mut key in device.list().await {
+                                for (i, pkesk) in wildcard.iter() {
+                                    let keyid = key.key_handle();
+                                    let public_key = key.public_key().await;
+                                    if pkesk.pk_algo() != public_key.pk_algo() {
+                                        log::trace!("skipping PKESK: \
+                                                     algorithm mismatch");
+                                        continue;
+                                    }
+
+                                    if ! key.decryption_capable().await {
+                                        log::trace!("{} cannot be used for decryption",
+                                                    key.fingerprint());
+                                        continue;
+                                    }
+
+                                    let mut abort = false;
+                                    let protection = key.locked().await;
+                                    log::trace!("{}'s protection: {:?}",
+                                                keyid, protection);
+                                    match protection {
+                                        // An unlocked key should work.
+                                        Protection::Unlocked => (),
+                                        // External protection may
+                                        // succeed, but it may block.
+                                        Protection::UnknownProtection(_) => (),
+                                        Protection::ExternalPassword(_) => (),
+                                        Protection::ExternalTouch(_) => (),
+                                        Protection::ExternalOther(_) => (),
+                                        protection @ Protection::Password(_) => {
+                                            log::trace!("{} is locked: {:?}",
+                                                        keyid, protection);
+                                            abort = true;
+                                        }
+                                    }
+                                    if ! key.available().await {
+                                        log::trace!("{} is unavailable",
+                                                    key.fingerprint());
+                                        abort = true;
+                                    }
+                                    if abort {
+                                        log::trace!("{} is inaccessible",
+                                                    keyid);
+                                        inaccessible.push(InaccessibleDecryptionKey {
+                                            key: KeyDescriptor {
+                                                server: KeyServer::new(
+                                                    ks.clone(),
+                                                    backend.id(),
+                                                    device.id(), key.id()),
+                                                key: key.public_key().await.clone(),
+                                            },
+                                            pkesk: pkesk.clone(),
+                                        });
+                                        continue;
+                                    }
+
+                                    if let Some((sym_algo, sk))
+                                        = key.decrypt_pkesk(pkesk).await
+                                    {
+                                        let fpr = key.fingerprint();
+                                        return Ok((*i, fpr, sym_algo, sk));
+                                    } else {
+                                        log::trace!("PKESK failed to decrypt \
+                                                     with {}", keyid);
+                                    }
+                                }
+                            }
+                        }
+                    }
                 }
 
                 if ! inaccessible.is_empty() {
@@ -361,8 +438,15 @@ impl keystore::Server for KeystoreServer {
                 let mut ok = results.get().get_result()?.init_ok();
 
                 ok.set_index(i as u32);
-                ok.set_fingerprint(&fingerprint);
-                ok.set_algo(u8::from(sym_algo));
+                ok.set_fingerprint(fingerprint.as_bytes());
+                ok.set_fingerprint_version(match fingerprint {
+                    Fingerprint::V4(_) => 4,
+                    Fingerprint::V6(_) => 6,
+                    Fingerprint::Unknown { version, .. } =>
+                        version.unwrap_or(0),
+                    _ => 0,
+                });
+                ok.set_algo(sym_algo.map(u8::from).unwrap_or(0));
                 ok.set_session_key(&sk);
 
                 Ok::<_, anyhow::Error>(())
@@ -399,8 +483,8 @@ impl keystore::backend::Server for BackendServer {
         Promise::ok(())
     }
 
-    fn list(&mut self, _params: keystore::backend::ListParams,
-            mut results: keystore::backend::ListResults)
+    fn devices(&mut self, _params: keystore::backend::DevicesParams,
+            mut results: keystore::backend::DevicesResults)
             -> ::capnp::capability::Promise<(), ::capnp::Error>
     {
         // Phase 1: Extract the parameters, and copy any required data
@@ -553,8 +637,8 @@ impl keystore::device::Server for DeviceServer {
         Promise::ok(())
     }
 
-    fn list(&mut self, _params: keystore::device::ListParams,
-            mut results: keystore::device::ListResults)
+    fn keys(&mut self, _params: keystore::device::KeysParams,
+            mut results: keystore::device::KeysResults)
         -> ::capnp::capability::Promise<(), ::capnp::Error>
     {
         // Phase 1: Extract the parameters, and copy any required data
